@@ -1,15 +1,15 @@
 """JSON utilities."""
+
 from base64 import b64decode, b64encode
+from io import StringIO
 import sys
+import warnings
 
+import dill
 import numpy as np
-import six
+import uncertainties
 
-try:
-    import dill
-    HAS_DILL = True
-except ImportError:
-    HAS_DILL = False
+HAS_DILL = True
 
 try:
     from pandas import DataFrame, Series, read_json
@@ -18,20 +18,7 @@ except ImportError:
     read_json = None
 
 
-def bindecode(val):
-    """b64decode wrapper, Python 2 and 3 version."""
-    return b64decode(six.b(val))
-
-
-if six.PY3:
-    def binencode(val):
-        """b64encode wrapper, Python 3 version."""
-        # b64encode result is /always/ UTF-8
-        return str(b64encode(val), 'utf-8')
-else:
-    def binencode(val):
-        """b64encode wrapper, Python 2 version."""
-        return str(b64encode(val))
+pyvers = f'{sys.version_info.major}.{sys.version_info.minor}'
 
 
 def find_importer(obj):
@@ -67,48 +54,46 @@ def encode4js(obj):
     """
     if isinstance(obj, DataFrame):
         return dict(__class__='PDataFrame', value=obj.to_json())
-    elif isinstance(obj, Series):
+    if isinstance(obj, Series):
         return dict(__class__='PSeries', value=obj.to_json())
-    elif isinstance(obj, np.ndarray):
+    if isinstance(obj, uncertainties.core.AffineScalarFunc):
+        return dict(__class__='UFloat', val=obj.nominal_value, err=obj.std_dev)
+    if isinstance(obj, np.ndarray):
         if 'complex' in obj.dtype.name:
             val = [(obj.real).tolist(), (obj.imag).tolist()]
         elif obj.dtype.name == 'object':
-            val = [encode4js(item) for item in obj['value']]
+            val = [encode4js(item) for item in obj]
         else:
             val = obj.flatten().tolist()
         return dict(__class__='NDArray', __shape__=obj.shape,
                     __dtype__=obj.dtype.name, value=val)
-    elif isinstance(obj, (np.float, np.int)):
+    if isinstance(obj, float):
         return float(obj)
-    elif isinstance(obj, six.string_types):
+    if isinstance(obj, int):
+        return int(obj)
+    if isinstance(obj, str):
         try:
             return str(obj)
         except UnicodeError:
             return obj
-    elif isinstance(obj, np.complex):
+    if isinstance(obj, complex):
         return dict(__class__='Complex', value=(obj.real, obj.imag))
-    elif isinstance(obj, (tuple, list)):
+    if isinstance(obj, (tuple, list)):
         ctype = 'List'
         if isinstance(obj, tuple):
             ctype = 'Tuple'
         val = [encode4js(item) for item in obj]
         return dict(__class__=ctype, value=val)
-    elif isinstance(obj, dict):
+    if isinstance(obj, dict):
         out = dict(__class__='Dict')
         for key, val in obj.items():
             out[encode4js(key)] = encode4js(val)
         return out
-    elif callable(obj):
-        val, importer = None, None
-        pyvers = "%d.%d" % (sys.version_info.major,
-                            sys.version_info.minor)
-        if HAS_DILL:
-            val = binencode(dill.dumps(obj))
-        else:
-            val = None
-            importer = find_importer(obj)
+    if callable(obj):
+        value = str(b64encode(dill.dumps(obj)), 'utf-8')
         return dict(__class__='Callable', __name__=obj.__name__,
-                    pyversion=pyvers, value=val, importer=importer)
+                    pyversion=pyvers, value=value,
+                    importer=find_importer(obj))
     return obj
 
 
@@ -118,9 +103,10 @@ def decode4js(obj):
         return obj
     out = obj
     classname = obj.pop('__class__', None)
+    if classname is None and isinstance(obj, dict):
+        classname = 'dict'
     if classname is None:
         return obj
-
     if classname == 'Complex':
         out = obj['value'][0] + 1j*obj['value'][1]
     elif classname in ('List', 'Tuple'):
@@ -141,17 +127,29 @@ def decode4js(obj):
             out = np.fromiter(obj['value'], dtype=obj['__dtype__'])
         out.shape = obj['__shape__']
     elif classname == 'PDataFrame' and read_json is not None:
-        out = read_json(obj['value'])
+        out = read_json(StringIO(obj['value']))
     elif classname == 'PSeries' and read_json is not None:
-        out = read_json(obj['value'], typ='series')
+        out = read_json(StringIO(obj['value']), typ='series')
+    elif classname == 'UFloat':
+        out = uncertainties.ufloat(obj['val'], obj['err'])
     elif classname == 'Callable':
-        out = val = obj['__name__']
-        pyvers = "%d.%d" % (sys.version_info.major,
-                            sys.version_info.minor)
-        if pyvers == obj['pyversion'] and HAS_DILL:
-            out = dill.loads(bindecode(obj['value']))
-        elif obj['importer'] is not None:
-            out = import_from(obj['importer'], val)
+        out = obj['__name__']
+        try:
+            out = import_from(obj['importer'], out)
+            unpacked = True
+        except (ImportError, AttributeError):
+            unpacked = False
+        if not unpacked:
+            spyvers = obj.get('pyversion', '?')
+            if not pyvers == spyvers:
+                msg = f"Could not unpack dill-encoded callable '{out}', saved with Python version {spyvers}"
+                warnings.warn(msg)
+
+            try:
+                out = dill.loads(b64decode(obj['value']))
+            except RuntimeError:
+                msg = f"Could not unpack dill-encoded callable '{out}`, saved with Python version {spyvers}"
+                warnings.warn(msg)
 
     elif classname in ('Dict', 'dict'):
         out = {}
