@@ -16,7 +16,7 @@ import lmfit
 
 from . import Minimizer, Parameter, Parameters, lineshapes
 from .confidence import conf_interval
-from .jsonutils import HAS_DILL, decode4js, encode4js
+from .jsonutils import decode4js, encode4js
 from .minimizer import MinimizerResult
 from .printfuncs import ci_report, fit_report, fitreport_html_table
 
@@ -145,7 +145,7 @@ def propagate_err(z, dz, option):
     if option not in ['real', 'imag', 'abs', 'angle']:
         raise ValueError(f"Invalid option ('{option}') for function 'propagate_err'.")
 
-    if z.shape != dz.shape:
+    if isinstance(dz, np.ndarray) and z.shape != dz.shape:
         raise ValueError(f"shape of z: {z.shape} != shape of dz: {dz.shape}")
 
     # Check the main vector for complex. Do nothing if real.
@@ -190,6 +190,20 @@ def propagate_err(z, dz, option):
     return err
 
 
+def coerce_arraylike(x):
+    """
+    coerce lists, tuples, and pandas Series, hdf5 Groups, etc to an
+    ndarray float64 or complex128, but leave other data structures
+    and objects unchanged
+    """
+    if isinstance(x, (list, tuple, Series)) or hasattr(x, '__array__'):
+        if np.isrealobj(x):
+            return np.asarray(x, dtype=np.float64)
+        if np.iscomplexobj(x):
+            return np.asarray(x, dtype=np.complex128)
+    return x
+
+
 class Model:
     """Create a model from a user-supplied model function."""
 
@@ -230,13 +244,18 @@ class Model:
 
         Notes
         -----
-        1. Parameter names are inferred from the function arguments, and a
-        residual function is automatically constructed.
-
-        2. The model function must return an array that will be the same
+        1. The model function must return an array that will be the same
         size as the data being modeled.
 
-        3. `nan_policy` sets what to do when a NaN or missing value is
+        2. Parameter names are inferred from the function arguments by default,
+        and a residual function is automatically constructed.
+
+        3. Specifying `independent_vars` here will explicitly name the
+        independent variables for the Model.  in contrast, `param_names` is
+        meant to help infer Parameter names for keyword arguments defined with
+        ``**kws`` in the Model function.
+
+        4. `nan_policy` sets what to do when a NaN or missing value is
         seen in the data. Should be one of:
 
            - `'raise'` : raise a `ValueError` (default)
@@ -280,6 +299,7 @@ class Model:
 
         self.opts = kws
         # the following has been changed from OrderedSet for the time being
+        self.independent_vars_defvals = {}
         self.param_hints = {}
         self._param_names = []
         self._parse_params()
@@ -305,17 +325,36 @@ class Model:
         """Save a Model for serialization.
 
         Note: like the standard-ish '__getstate__' method but not really
-        useful with Pickle.
+        useful with Pickle, and only useful with dill.
 
+        This, and the companion function _buildmodel to use this serialized model
+        now supports versions of 'state'.
+
+        State Version History:
+          original: up to and including version 1.2.2:
+             state is a tuple of length 9:
+                (self.func.__name__, funcdef, self._name, self._prefix,
+                self.independent_vars, self._param_root_names,
+                self.param_hints, self.nan_policy, self.opts)
+             with opts used in the version 1.2 sense
+          version 1.2.3 and beyond:
+             state is a dict with a 'version' keyword and all other
+             values from the original state in key/value pairs.
+             The initial value for 'version' is '2'.
         """
-        funcdef = None
-        if HAS_DILL:
-            funcdef = self.func
+        funcdef = self.func
         if self.func.__name__ == '_eval':
             funcdef = self.expr
-        state = (self.func.__name__, funcdef, self._name, self._prefix,
-                 self.independent_vars, self._param_root_names,
-                 self.param_hints, self.nan_policy, self.opts)
+        state = dict(version='2',
+                     funcname=self.func.__name__,
+                     funcdef=funcdef,
+                     name=self._name,
+                     prefix=self._prefix,
+                     independent_vars=self.independent_vars,
+                     param_root_names=self._param_root_names,
+                     param_hints=self.param_hints,
+                     nan_policy=self.nan_policy,
+                     opts=self.opts)
         return (state, None, None)
 
     def _set_state(self, state, funcdefs=None):
@@ -482,40 +521,62 @@ class Model:
         # 2. modern, best-practice approach: use inspect.signature
         else:
             pos_args = []
+            default_vals = {}
+            indep_vars = []
             sig = inspect.signature(self.func)
             for fnam, fpar in sig.parameters.items():
                 if fpar.kind == fpar.VAR_KEYWORD:
                     keywords_ = fnam
-                elif fpar.kind == fpar.POSITIONAL_OR_KEYWORD:
-                    if fpar.default == fpar.empty:
+                elif fpar.kind in (fpar.POSITIONAL_ONLY, fpar.POSITIONAL_OR_KEYWORD):
+                    default_vals[fnam] = fpar.default
+                    if (isinstance(fpar.default, (float, int, complex))
+                       and not isinstance(fpar.default, bool)):
+                        kw_args[fnam] = fpar.default
+                    elif fpar.default == fpar.empty:
                         pos_args.append(fnam)
                     else:
                         kw_args[fnam] = fpar.default
+                        indep_vars.append(fnam)
                 elif fpar.kind == fpar.VAR_POSITIONAL:
                     raise ValueError(f"varargs '*{fnam}' is not supported")
         # inspection done
 
         self._func_haskeywords = keywords_ is not None
-        self._func_allargs = pos_args + list(kw_args.keys())
-        allargs = self._func_allargs
+        self._func_allargs = list(default_vals.keys())
+        for key in kw_args:
+            if key not in self._func_allargs:
+                self._func_allargs.append(key)
 
-        if len(allargs) == 0 and keywords_ is not None:
+        if len(self._func_allargs) == 0 and keywords_ is not None:
             return
 
+        self.independent_vars_defvals = {}
         # default independent_var = 1st argument
         if self.independent_vars is None:
-            self.independent_vars = [pos_args[0]]
+            self.independent_vars = [pos_args.pop(0)]
+            # default values for independent variables
+            for vnam in indep_vars:
+                dval = default_vals[vnam]
+                if vnam in self.opts:
+                    dval = self.opts[vnam]
+                self.independent_vars_defvals[vnam] = dval
+                if vnam not in self.independent_vars:
+                    self.independent_vars.append(vnam)
 
         # default param names: all positional args
         # except independent variables
         self.def_vals = {}
         might_be_param = []
         if self._param_root_names is None:
-            self._param_root_names = pos_args[:]
+            self._param_root_names = []
+            for pname in pos_args:
+                if pname not in self.independent_vars:
+                    self._param_root_names.append(pname)
             for key, val in kw_args.items():
                 if (not isinstance(val, bool) and
                         isinstance(val, (float, int))):
-                    self._param_root_names.append(key)
+                    if key not in self._param_root_names:
+                        self._param_root_names.append(key)
                     self.def_vals[key] = val
                 elif val is None:
                     might_be_param.append(key)
@@ -536,14 +597,15 @@ class Model:
         if self._prefix is None:
             self._prefix = ''
         names = [f"{self._prefix}{pname}" for pname in self._param_root_names]
+
         # check variables names for validity
         # The implicit magic in fit() requires us to disallow some
         fname = self.func.__name__
         for arg in self.independent_vars:
-            if arg not in allargs or arg in self._forbidden_args:
+            if arg not in self._func_allargs or arg in self._forbidden_args:
                 raise ValueError(self._invalid_ivar % (arg, fname))
         for arg in names:
-            if (self._strip_prefix(arg) not in allargs or
+            if (self._strip_prefix(arg) not in self._func_allargs or
                     arg in self._forbidden_args):
                 raise ValueError(self._invalid_par % (arg, fname))
         # the following as been changed from OrderedSet for the time being.
@@ -665,9 +727,17 @@ class Model:
         params = Parameters()
 
         def setpar(par, val):
-            if isinstance(val, (float, int)):
-                val = {'value': val}
-            par.set(**val)
+            # val is expected to be float-like or a dict: must have 'value' or 'expr' key
+            if isinstance(val, dict):
+                dval = val
+            elif np.iscomplex(val) or isinstance(val, complex):
+                dval = {'value': val.real}
+            else:
+                dval = {'value': float(val)}
+            if len(dval) < 1 or not ('value' in dval or 'expr' in dval):
+                raise TypeError(f'Invalid parameter value for {par}: {val}')
+
+            par.set(**dval)
 
         # make sure that all named parameters are in params
         for name in self.param_names:
@@ -694,7 +764,7 @@ class Model:
             if basename in kwargs:
                 setpar(par, kwargs[basename])
             if name in kwargs:
-                setpar(par, kwargs[basename])
+                setpar(par, kwargs[name])
             params.add(par)
             if verbose:
                 print(f' - Adding parameter "{name}"')
@@ -721,6 +791,20 @@ class Model:
             if name not in self._param_names:
                 self._param_names.append(name)
 
+        # check for parameters that were initially flagged as independent
+        # variables because the function signature used "key=None", "key=True",
+        # or "key=False": these could actually be variables
+        for key, val in kwargs.items():
+            if key in params:
+                continue
+            if key in self.independent_vars:
+                dxval = self.independent_vars_defvals.get(key, inspect._empty)
+                if dxval is None or isinstance(dxval, bool):
+                    name = f"{self._prefix}{key}"
+                    par = Parameter(name=name)
+                    setpar(par, val)
+                    params.add(par)
+
         for p in params.values():
             p._delay_asteval = False
         return params
@@ -793,11 +877,11 @@ class Model:
 
         diff = model - data
 
-        if diff.dtype == complex:
+        if diff.dtype is complex:
             # data/model are complex
             diff = diff.ravel().view(float)
             if weights is not None:
-                if weights.dtype == complex:
+                if weights.dtype is complex:
                     # weights are complex
                     weights = weights.ravel().view(float)
                 else:
@@ -805,7 +889,7 @@ class Model:
                     weights = (weights + 1j * weights).ravel().view(float)
         if weights is not None:
             diff *= weights
-        return np.asarray(diff).ravel()  # for compatibility with pandas.Series
+        return diff
 
     def _strip_prefix(self, name):
         npref = len(self._prefix)
@@ -820,8 +904,9 @@ class Model:
         if kwargs is None:
             kwargs = {}
         out = {}
-        out.update(self.opts)
-
+        for key, val in self.independent_vars_defvals.items():
+            if val is not inspect._empty:
+                out[key] = val
         # 0: if a keyword argument is going to overwrite a parameter,
         #    save that value so it can be restored before returning
         saved_values = {}
@@ -850,10 +935,12 @@ class Model:
                         out[name] = params[fullname].value
 
         # 3. kwargs might directly update function arguments
+        validnames = [ivar for ivar in self.independent_vars]
+        validnames.extend(self._func_allargs)
         for name, val in kwargs.items():
             if strip:
                 name = self._strip_prefix(name)
-            if name in self._func_allargs or self._func_haskeywords:
+            if name in validnames or self._func_haskeywords:
                 out[name] = val
 
         # 4. finally, reset any values that have overwritten parameter values
@@ -861,6 +948,12 @@ class Model:
             params[name].value = val
         return out
 
+    def post_fit(self, fitresult):
+        """function that is called just after fit, can be overloaded by
+        subclasses to add non-fitting 'calculated parameters'
+        """
+        pass
+
     def _make_all_args(self, params=None, **kwargs):
         """Generate **all** function args for all functions."""
         args = {}
@@ -901,7 +994,7 @@ class Model:
         values may be Python `float`, `int`, or  `complex` values.
 
         """
-        return self.func(**self.make_funcargs(params, kwargs))
+        return coerce_arraylike(self.func(**self.make_funcargs(params, kwargs)))
 
     @property
     def components(self):
@@ -932,7 +1025,8 @@ class Model:
 
     def fit(self, data, params=None, weights=None, method='leastsq',
             iter_cb=None, scale_covar=True, verbose=False, fit_kws=None,
-            nan_policy=None, calc_covar=True, max_nfev=None, **kwargs):
+            nan_policy=None, calc_covar=True, max_nfev=None,
+            coerce_farray=True, **kwargs):
         """Fit the model to the data using the supplied Parameters.
 
         Parameters
@@ -966,6 +1060,11 @@ class Model:
         max_nfev : int or None, optional
             Maximum number of function evaluations (default is None). The
             default value depends on the fitting method.
+        coerce_farray : bool, optional
+            Whether to coerce data and independent data to be ndarrays
+            with dtype of float64 (or complex128).  If set to False, data
+            and independent data are not coerced at all, but the output of
+            the model function will be. (default is True)
         **kwargs : optional
             Arguments to pass to the model function, possibly overriding
             parameters.
@@ -1051,28 +1150,19 @@ class Model:
         # If independent_vars and data are alignable (pandas), align them,
         # and apply the mask from above if there is one.
         for var in self.independent_vars:
-            if not np.isscalar(kwargs[var]):
-                kwargs[var] = _align(kwargs[var], mask, data)
+            if var not in params:
+                if var not in kwargs:
+                    raise ValueError(f"'Missing independent variable '{var}'")
+                if not np.isscalar(kwargs[var]):
+                    kwargs[var] = _align(kwargs[var], mask, data)
 
-        def coerce_arraylike(x):
-            """
-            coerce lists, tuples, and pandas Series to float64 or complex128,
-            but leave other ndarrays of different dtypes and objects unchanged
-            """
-            if isinstance(x, (list, tuple, Series)):
-                if np.isrealobj(x):
-                    return np.asfarray(x)
-                elif np.iscomplexobj(x):
-                    return np.asarray(x, dtype='complex128')
-            return x
-
-        # coerce data and independent variable(s) that are 'array-like' (list,
-        # tuples, pandas Series) to float64/complex128. Note: this will not
-        # alter the dtype of data or independent variables that are already
-        # ndarrays but with dtype other than float64/complex128.
-        data = coerce_arraylike(data)
-        for var in self.independent_vars:
-            kwargs[var] = coerce_arraylike(kwargs[var])
+        if coerce_farray:
+            # coerce data and independent variable(s) that are 'array-like' (list,
+            # tuples, pandas Series) to float64/complex128.
+            data = coerce_arraylike(data)
+            for var in self.independent_vars:
+                if var not in params and var in kwargs:
+                    kwargs[var] = coerce_arraylike(kwargs[var])
 
         if fit_kws is None:
             fit_kws = {}
@@ -1197,6 +1287,13 @@ class CompositeModel(Model):
         out.update(self.right.eval_components(**kwargs))
         return out
 
+    def post_fit(self, fitresult):
+        """function that is called just after fit, can be overloaded by
+        subclasses to add non-fitting 'calculated parameters'
+        """
+        self.left.post_fit(fitresult)
+        self.right.post_fit(fitresult)
+
     @property
     def param_names(self):
         """Return parameter names for composite model."""
@@ -1271,28 +1368,50 @@ def _buildmodel(state, funcdefs=None):
         fcn = getattr(lineshapes, fname, None)
         if callable(fcn):
             known_funcs[fname] = fcn
-    if funcdefs is not None:
+    if funcdefs is None:
+        funcdefs = {}
+    else:
         known_funcs.update(funcdefs)
 
     left, right, op = state
     if op is None and right is None:
-        (fname, fcndef, name, prefix, ivars, pnames,
-         phints, nan_policy, opts) = left
-        if not callable(fcndef) and fname in known_funcs:
-            fcndef = known_funcs[fname]
+        if isinstance(left, tuple) and len(left) == 9:
+            (fname, func, name, prefix, ivars, pnames,
+             phints, nan_policy, opts) = left
+        elif isinstance(left, dict) and 'version' in left:
+            # for future-proofing, we could add "if left['version'] == '2':"
+            # here to cover cases when 'version' changes
+            fname = left.get('funcname', None)
+            func = left.get('funcdef', None)
+            name = left.get('name', None)
+            prefix = left.get('prefix', None)
+            ivars = left.get('indepedendent_vars', None)
+            pnames = left.get('param_root_names', None)
+            phints = left.get('param_hints', None)
+            nan_policy = left.get('nan_policy', None)
+            opts = left.get('opts', None)
+        else:
+            raise ValueError("Cannot restore Model: unrecognized state data")
 
-        if fcndef is None:
+        # if the function definition was passed in, use that!
+        if fname in funcdefs and fname != '_eval':
+            func = funcdefs[fname]
+
+        if not callable(func) and fname in known_funcs:
+            func = known_funcs[fname]
+
+        if func is None:
             raise ValueError("Cannot restore Model: model function not found")
 
-        if fname == '_eval' and isinstance(fcndef, str):
+        if fname == '_eval' and isinstance(func, str):
             from .models import ExpressionModel
-            model = ExpressionModel(fcndef, name=name,
+            model = ExpressionModel(func, name=name,
                                     independent_vars=ivars,
                                     param_names=pnames,
                                     nan_policy=nan_policy, **opts)
 
         else:
-            model = Model(fcndef, name=name, prefix=prefix,
+            model = Model(func, name=name, prefix=prefix,
                           independent_vars=ivars, param_names=pnames,
                           nan_policy=nan_policy, **opts)
 
@@ -1439,6 +1558,8 @@ class ModelResult(Minimizer):
         self.userkws.update(kwargs)
         self.init_fit = self.model.eval(params=self.params, **self.userkws)
         _ret = self.minimize(method=self.method)
+        self.model.post_fit(_ret)
+        _ret.params.create_uvars(covar=_ret.covar)
 
         for attr in dir(_ret):
             if not attr.startswith('_'):
@@ -1447,14 +1568,16 @@ class ModelResult(Minimizer):
                 except AttributeError:
                     pass
 
-        if self.data is not None and len(self.data) > 1:
-            sstot = ((self.data - self.data.mean())**2).sum()
-            if isinstance(self.residual, np.ndarray) and len(self.residual) > 1:
-                self.rsquared = 1.0 - (self.residual**2).sum()/max(tiny, sstot)
-
         self.init_values = self.model._make_all_args(self.init_params)
         self.best_values = self.model._make_all_args(_ret.params)
         self.best_fit = self.model.eval(params=_ret.params, **self.userkws)
+        if (self.data is not None and len(self.data) > 1
+           and isinstance(self.best_fit, np.ndarray)
+           and len(self.best_fit) > 1):
+            dat = coerce_arraylike(self.data)
+            resid = ((dat - self.best_fit)**2).sum()
+            sstot = ((dat - dat.mean())**2).sum()
+            self.rsquared = 1.0 - resid/max(tiny, sstot)
 
     def eval(self, params=None, **kwargs):
         """Evaluate model function.
@@ -1501,7 +1624,7 @@ class ModelResult(Minimizer):
             params = self.params
         return self.model.eval_components(params=params, **userkws)
 
-    def eval_uncertainty(self, params=None, sigma=1, **kwargs):
+    def eval_uncertainty(self, params=None, sigma=1, dscale=0.01, **kwargs):
         """Evaluate the uncertainty of the *model function*.
 
         This can be used to give confidence bands for the model from the
@@ -1513,6 +1636,8 @@ class ModelResult(Minimizer):
             Parameters, defaults to ModelResult.params.
         sigma : float, optional
             Confidence level, i.e. how many sigma (default is 1).
+        dscale : float, optional
+            Scale for derivative steps (default is 0.01).
         **kwargs : optional
             Values of options, independent variables, etcetera.
 
@@ -1533,10 +1658,14 @@ class ModelResult(Minimizer):
            < 1, it is interpreted as the probability itself. That is,
            ``sigma=1`` and ``sigma=0.6827`` will give the same results,
            within precision errors.
-        3. Also sets attributes of `dely` for the uncertainty of the model
+        3. The derivatives are calculated by stepping each Parameter from its best value to
+           to +/- stderr*dscale, where `dscale` can be passed in and defaults to 0.01.
+        4. Sets attributes of `dely` for the uncertainty of the model
            (which will be the same as the array returned by this method) and
            `dely_comps`, a dictionary of `dely` for each component.
-
+        5. Sets the attribute of `dely_predicted` for the 'predicted interval', the sigma-scaled
+           quadrature sum of the uncertainty interval `dely` and reduced chi-square. This should
+           give an idea of the expected range in the data.
 
         Examples
         --------
@@ -1556,25 +1685,28 @@ class ModelResult(Minimizer):
 
         nvarys = self.nvarys
         # ensure fjac and df2 are correct size if independent var updated by kwargs
-        ndata = self.model.eval(params, **userkws).size
+        feval = self.model.eval(params, **userkws)
+        ndata = len(feval.view('float64'))        # allows feval to be complex
         covar = self.covar
         if any(p.stderr is None for p in params.values()):
             return np.zeros(ndata)
 
-        fjac = {'0': np.zeros((nvarys, ndata))}  # '0' signify 'Full', an invalid prefix
-        df2 = {'0': np.zeros(ndata)}
+        # '0' would be an invalid prefix, here signifying 'Full'
+        fjac = {'0': np.zeros((nvarys, ndata), dtype='float64')}
+        df2 = {'0': np.zeros(ndata, dtype='float64')}
 
-        for comp in self.components:
+        for comp in self.model.components:
             label = comp.prefix if len(comp.prefix) > 1 else comp._name
-            fjac[label] = np.zeros((nvarys, ndata))
-            df2[label] = np.zeros(ndata)
+            fjac[label] = np.zeros((nvarys, ndata), dtype='float64')
+            df2[label] = np.zeros(ndata, dtype='float64')
 
         # find derivative by hand!
         pars = params.copy()
         for i in range(nvarys):
             pname = self.var_names[i]
-            val0 = pars[pname].value
-            dval = pars[pname].stderr/3.0
+            val0 = params[pname].value
+            dval = params[pname].stderr*dscale
+
             pars[pname].value = val0 + dval
             res1 = {'0': self.model.eval(pars, **userkws)}
             res1.update(self.model.eval_components(params=pars, **userkws))
@@ -1585,7 +1717,8 @@ class ModelResult(Minimizer):
 
             pars[pname].value = val0
             for key in fjac:
-                fjac[key][i] = (res1[key] - res2[key]) / (2*dval)
+                fjac[key][i] = (res1[key].view('float64')
+                                - res2[key].view('float64')) / (2*dval)
 
         for i in range(nvarys):
             for j in range(nvarys):
@@ -1598,7 +1731,15 @@ class ModelResult(Minimizer):
             prob = erf(sigma/np.sqrt(2))
 
         scale = t.ppf((prob+1)/2.0, self.ndata-nvarys)
-        self.dely = scale * np.sqrt(df2.pop('0'))
+
+        # for complex data, convert back to real/imag pairs
+        if feval.dtype in ('complex64', 'complex128'):
+            for key in fjac:
+                df2[key] = df2[key][0::2] + 1j * df2[key][1::2]
+
+        df2_total = df2.pop('0')
+        self.dely = scale * np.sqrt(df2_total)
+        self.dely_predicted = scale * np.sqrt(df2_total + self.redchi)
 
         self.dely_comps = {}
         for key in df2:
@@ -1687,7 +1828,7 @@ class ModelResult(Minimizer):
         report = fitreport_html_table(self, show_correl=show_correl,
                                       min_correl=min_correl)
         modname = self.model._reprstring(long=True)
-        return f"<h2> Model</h2> {modname} {report}"
+        return f"<h2>Fit Result</h2> <p>Model: {modname}</p> {report}"
 
     def summary(self):
         """Return a dictionary with statistics and attributes of a ModelResult.
@@ -1760,12 +1901,11 @@ class ModelResult(Minimizer):
         loads, json.dumps
 
         """
-        out = {'__class__': 'lmfit.ModelResult', '__version__': '1',
+        out = {'__class__': 'lmfit.ModelResult', '__version__': '2',
                'model': encode4js(self.model._get_state())}
-        pasteval = self.params._asteval
-        out['params'] = [p.__getstate__() for p in self.params.values()]
-        out['unique_symbols'] = {key: encode4js(pasteval.symtable[key])
-                                 for key in pasteval.user_defined_symbols()}
+
+        for attr in ('params', 'init_params'):
+            out[attr] = getattr(self, attr).dumps()
 
         for attr in ('aborted', 'aic', 'best_values', 'bic', 'chisqr',
                      'ci_out', 'col_deriv', 'covar', 'errorbars', 'flatchain',
@@ -1780,7 +1920,6 @@ class ModelResult(Minimizer):
                 continue
             if isinstance(val, np.bool_):
                 val = bool(val)
-
             out[attr] = encode4js(val)
 
         val = out.get('message', '')
@@ -1822,7 +1961,7 @@ class ModelResult(Minimizer):
         funcdefs : dict, optional
             Dictionary of custom function names and definitions.
         **kws : optional
-            Keyword arguments that are passed to `json.dumps`.
+            Keyword arguments that are passed to `json.loads`.
 
         Returns
         -------
@@ -1845,16 +1984,31 @@ class ModelResult(Minimizer):
         # model
         self.model = _buildmodel(decode4js(modres['model']), funcdefs=funcdefs)
 
-        # params
-        for target in ('params', 'init_params'):
-            state = {'unique_symbols': modres['unique_symbols'], 'params': []}
-            for parstate in modres['params']:
-                _par = Parameter(name='')
-                _par.__setstate__(parstate)
-                state['params'].append(_par)
-            _params = Parameters()
-            _params.__setstate__(state)
-            setattr(self, target, _params)
+        if funcdefs:
+            # Remove model function so as not pass it into the _asteval.symtable
+            funcdefs.pop(self.model.func.__name__, None)
+
+        # how params are saved was changed with version 2:
+        modres_vers = modres.get('__version__', '1')
+        if modres_vers == '1':
+            for target in ('params', 'init_params'):
+                state = {'unique_symbols': modres['unique_symbols'], 'params': []}
+                for parstate in modres['params']:
+                    _par = Parameter(name='')
+                    _par.__setstate__(parstate)
+                    state['params'].append(_par)
+                _params = Parameters(usersyms=funcdefs)
+                _params.__setstate__(state)
+                setattr(self, target, _params)
+
+        elif modres_vers == '2':
+            for target in ('params', 'init_params'):
+                _pars = Parameters()
+                _pars.loads(modres[target])
+                if funcdefs:
+                    for key, val in funcdefs.items():
+                        _pars._asteval.symtable[key] = val
+                setattr(self, target, _pars)
 
         for attr in ('aborted', 'aic', 'best_fit', 'best_values', 'bic',
                      'chisqr', 'ci_out', 'col_deriv', 'covar', 'data',
@@ -1876,9 +2030,14 @@ class ModelResult(Minimizer):
             if par is not None:
                 par.correl = par.stderr = None
                 par.value = par.init_value = self.init_values[parname]
+
         self.init_fit = self.model.eval(self.init_params, **self.userkws)
         self.result = MinimizerResult()
         self.result.params = self.params
+
+        if self.errorbars and self.covar is not None:
+            self.uvars = self.result.params.create_uvars(covar=self.covar)
+
         self.init_vals = list(self.init_values.items())
         return self
 
@@ -2027,7 +2186,6 @@ class ModelResult(Minimizer):
 
         if yerr is None and self.weights is not None:
             yerr = 1.0/self.weights
-
         if yerr is not None:
             ax.errorbar(x_array, reduce_complex(self.data),
                         yerr=propagate_err(self.data, yerr, parse_complex),
